use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007003
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Ahmad
Chris
Compile
David
Dist
Etheridge
Fredric
Golden
Graham
Gruenauer
Harley
Jesse
Jérôme
Karen
Kent
Knop
Luehrs
Marcel
Mengué
Olivier
Peter
Pig
Plugin
Quelin
Randy
Ricardo
SIGNES
Shangov
Stauner
Test
Weyl
Zawawi
Zilla
ahmad
cweyl
dagolden
dolmen
doy
ether
fayland
haarg
hanekomu
harleypig
irc
jquelin
kentnl
lib
pshangov
rjbs
rwstauner
